@@ -182,20 +182,54 @@ class Trainer:
         if self.config.sample_weight_mode is not None:
             return self.config.sample_weight_mode
 
+    def _check_shapes(self, train_dataset, val_dataset):
+        """Check that the shapes of the train and validation datasets are the same"""
+        if isinstance(train_dataset, tf.data.Dataset) and isinstance(val_dataset, tf.data.Dataset):
+            train_shape = train_dataset.element_spec
+            val_shape = val_dataset.element_spec
+            if len(train_shape) > 4 or len(val_shape) > 4:
+                raise ValueError("The shape of the input data must be lower than 4 dimensions")
+            if train_shape != val_shape:
+                raise ValueError(
+                    f"Shapes of train and validation datasets are different. Train: {train_shape}, Val: {val_shape}"
+                )
+        elif isinstance(train_dataset, tuple) and isinstance(val_dataset, tuple):
+
+            if len(train_dataset) > 4 or len(val_dataset) > 4:
+                raise ValueError("The shape of the input data must be lower than 4 dimensions")
+            if len(train_dataset) != len(val_dataset):
+                raise ValueError(
+                    f"Shapes of train and validation datasets are different."
+                )
+
+    def add_callback(self, callback: tf.keras.callbacks.Callback):
+        """Add a callback to the model
+
+        Args:
+            callback:  Callback to add to the model
+        """
+        self._callbacks.append(callback)
+
     def _get_input_shape(self):
         """Get the input shape of input dataset"""
 
         if isinstance(self.train_dataset, tf.data.Dataset):
             for a in self.train_dataset.take(1):
-                self.input_shape = (None, *a[0].shape[1:])
+               
+                arr = a[0] if isinstance(a[0], tuple) else a
+                self.input_shape = (None, *arr[0].shape[1:])
         else:
-            self.input_shape = (None, *self.train_dataset[0].shape[1:])
+            arr = self.train_dataset[0] if isinstance(self.train_dataset[0], tuple) else self.train_dataset
+            self.input_shape = (None, *arr[0].shape[1:])
         return self.input_shape
 
     def _save_model(self):
         """Save the model by loading best checkpoint if available and saving it to mlflow or local path"""
+        if self.config.save_model is not None:
+            self._out_path = self.config.save_model
+            os.makedirs(self._out_path, exist_ok=True)
 
-        if self._model_checkpoint is not None:
+        elif self._model_checkpoint is not None:
             self.model.load_weights(self._model_checkpoint)
             if self._log:
                 run = mlflow.last_active_run()
@@ -231,6 +265,7 @@ class Trainer:
         except NotImplementedError:  # there's custom layer
             raise
 
+
     def _get_x_y_val(self, batch_size: int) -> tf.data.Dataset:
         """Get the x and y for training based on the format of the dataset"""
 
@@ -615,6 +650,9 @@ class Trainer:
             for val in self.val_dataset:
                 if not isinstance(val, np.ndarray):
                     raise TypeError("val_dataset must be a tuple of numpy arrays")
+
+        self._check_shapes(self.train_dataset, self.val_dataset)
+
         if not (
             isinstance(self.config_params, dict) or isinstance(self.config_params, str)
         ):
@@ -647,7 +685,8 @@ class Trainer:
         self._validate_config_file()
 
         self._callbacks = self._get_callbacks()
-        self._input_shape = self._get_input_shape()
+        #self._input_shape = self._get_input_shape()
         self._sample_weight_mode = self._get_sample_weight_mode()
 
         self._trainer_initialized = True
+
